// University of Illinois/NCSA
// Open Source License
//
// Copyright (c) 2013-2015, Advanced Micro Devices, Inc.
// All rights reserved.
//
// Developed by:
//
//     HSA Team
//
//     Advanced Micro Devices, Inc
//
//     www.amd.com
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal with
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
//     * Redistributions of source code must retain the above copyright notice,
//       this list of conditions and the following disclaimers.
//
//     * Redistributions in binary form must reproduce the above copyright notice,
//       this list of conditions and the following disclaimers in the
//       documentation and/or other materials provided with the distribution.
//
//     * Neither the names of the LLVM Team, University of Illinois at
//       Urbana-Champaign, nor the names of its contributors may be used to
//       endorse or promote products derived from this Software without specific
//       prior written permission.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
// FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
// CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE
// SOFTWARE.

class InstValidator : public PropValidator
{
private:

    // Autogenerated accessors for BRIG properties
    #include "HSAILBrigPropsFastAcc_gen.hpp"

public:
    InstValidator(unsigned model, unsigned profile) : PropValidator(model, profile) {}
    void validateInst(Inst inst) const;

public:
    unsigned getOperand0Attr(Inst inst) const;

private:

public:
    unsigned getOperand1Attr(Inst inst) const;

private:

public:
    unsigned getOperand2Attr(Inst inst) const;

private:

public:
    unsigned getOperand3Attr(Inst inst) const;

private:

public:
    unsigned getOperand4Attr(Inst inst) const;

private:

public:
    unsigned getOperand5Attr(Inst inst) const;

private:

public:
    unsigned getRoundAttr(Inst inst) const;

private:

public:
    unsigned getWidthAttr(Inst inst) const;

private:

public:
    unsigned getOperandAttr(Inst inst, unsigned operandIdx) const
    {
        switch(operandIdx)
        {
        case 0: return getOperand0Attr(inst);
        case 1: return getOperand1Attr(inst);
        case 2: return getOperand2Attr(inst);
        case 3: return getOperand3Attr(inst);
        case 4: return getOperand4Attr(inst);
        case 5: return getOperand5Attr(inst);
        default:
            assert(false);
            return OPERAND_ATTR_INVALID;
        }
    }


private:
    static unsigned ALIGN_VALUES_ANY[];
    static unsigned ATMOP_VALUES_ADD_MIN_MAX[];
    static unsigned ATMOP_VALUES_MIN_MAX[];
    static unsigned CONST_VALUES_NONE[];
    static unsigned EQCLASS_VALUES_ANY[];
    static unsigned FTZ_VALUES_ANY[];
    static unsigned MEMORD_VALUES_ANY[];
    static unsigned MEMSCP_VALUES_WV_WG[];
    static unsigned OPERAND_VALUES_ADDRSEG[];
    static unsigned OPERAND_VALUES_IMM[];
    static unsigned OPERAND_VALUES_REG_IMM[];
    static unsigned OPERAND_VALUES_REG_VECTOR_IMM[];
    static unsigned OPERAND_VALUES_NULL[];
    static unsigned OPERAND_VALUES_REG[];
    static unsigned OPERAND_VALUES_REG_VECTOR[];
    static unsigned OPERAND_VALUES_VEC4[];
    static unsigned PACK_VALUES_NONE[];
    static unsigned ROUND_VALUES_NONE[];
    static unsigned SEGMENT_VALUES_GCN[];
    static unsigned SEGMENT_VALUES_GROUP[];
    static unsigned TYPE_VALUES_U_S_F_B128[];
    static unsigned TYPE_VALUES_B32[];
    static unsigned TYPE_VALUES_B64[];
    static unsigned TYPE_VALUES_F[];
    static unsigned TYPE_VALUES_S32_U32_F[];
    static unsigned TYPE_VALUES_F32[];
    static unsigned TYPE_VALUES_F32_F64[];
    static unsigned TYPE_VALUES_S32_U32_F32[];
    static unsigned TYPE_VALUES_F64[];
    static unsigned TYPE_VALUES_U32[];
    static unsigned TYPESIZE_VALUES_ATOMIC[];
    static unsigned WIDTH_VALUES_ANY1[];
    static unsigned WIDTH_VALUES_NONE[];

private:
    static bool check_align_values_any(unsigned val);
    static bool check_atmop_values_add_min_max(unsigned val);
    static bool check_atmop_values_min_max(unsigned val);
    static bool check_const_values_none(unsigned val);
    static bool check_ftz_values_any(unsigned val);
    static bool check_memord_values_any(unsigned val);
    static bool check_memscp_values_wv_wg(unsigned val);
    static bool check_pack_values_none(unsigned val);
    static bool check_round_values_none(unsigned val);
    static bool check_segment_values_gcn(unsigned val);
    static bool check_segment_values_group(unsigned val);
    static bool check_type_values_u_s_f_b128(unsigned val);
    static bool check_type_values_b32(unsigned val);
    static bool check_type_values_b64(unsigned val);
    static bool check_type_values_f(unsigned val);
    static bool check_type_values_s32_u32_f(unsigned val);
    static bool check_type_values_f32(unsigned val);
    static bool check_type_values_f32_f64(unsigned val);
    static bool check_type_values_s32_u32_f32(unsigned val);
    static bool check_type_values_f64(unsigned val);
    static bool check_type_values_u32(unsigned val);
    static bool check_width_values_any1(unsigned val);
    static bool check_width_values_none(unsigned val);

private:
    template<class T> bool req_d0_s1(T inst) const;
    template<class T> bool req_d0_s1_s2(T inst) const;
    template<class T> bool req_d0_s1_s2_s3(T inst) const;
    template<class T> bool req_gcn_append_consume(T inst) const;
    template<class T> bool req_gcn_atomic(T inst) const;
    template<class T> bool req_gcn_atomic_noret(T inst) const;
    template<class T> bool req_gcn_b4xchg(T inst) const;
    template<class T> bool req_gcn_bfm(T inst) const;
    template<class T> bool req_gcn_div_relaxed(T inst) const;
    template<class T> bool req_gcn_div_relaxed_narrow(T inst) const;
    template<class T> bool req_gcn_fldexp(T inst) const;
    template<class T> bool req_gcn_frexp_exp(T inst) const;
    template<class T> bool req_gcn_frexp_mant(T inst) const;
    template<class T> bool req_gcn_ld(T inst) const;
    template<class T> bool req_gcn_mads(T inst) const;
    template<class T> bool req_gcn_madu(T inst) const;
    template<class T> bool req_gcn_min_max(T inst) const;
    template<class T> bool req_gcn_min_max_med3(T inst) const;
    template<class T> bool req_gcn_mqsad(T inst) const;
    template<class T> bool req_gcn_mqsad4(T inst) const;
    template<class T> bool req_gcn_msad(T inst) const;
    template<class T> bool req_gcn_qsad(T inst) const;
    template<class T> bool req_gcn_region_alloc(T inst) const;
    template<class T> bool req_gcn_slp_prt(T inst) const;
    template<class T> bool req_gcn_st(T inst) const;
    template<class T> bool req_gcn_trig_preop(T inst) const;

}; // class InstValidator

unsigned InstValidator::ALIGN_VALUES_ANY[] = {
    BRIG_ALIGNMENT_1,
    BRIG_ALIGNMENT_128,
    BRIG_ALIGNMENT_16,
    BRIG_ALIGNMENT_2,
    BRIG_ALIGNMENT_256,
    BRIG_ALIGNMENT_32,
    BRIG_ALIGNMENT_4,
    BRIG_ALIGNMENT_64,
    BRIG_ALIGNMENT_8
};

unsigned InstValidator::ATMOP_VALUES_ADD_MIN_MAX[] = {
    BRIG_ATOMIC_ADD,
    BRIG_ATOMIC_MAX,
    BRIG_ATOMIC_MIN
};

unsigned InstValidator::ATMOP_VALUES_MIN_MAX[] = {
    BRIG_ATOMIC_MAX,
    BRIG_ATOMIC_MIN
};

unsigned InstValidator::CONST_VALUES_NONE[] = {
    0
};

unsigned InstValidator::EQCLASS_VALUES_ANY[] = {
    EQCLASS_VAL_ANY
};

unsigned InstValidator::FTZ_VALUES_ANY[] = {
    0,
    1
};

unsigned InstValidator::MEMORD_VALUES_ANY[] = {
    BRIG_MEMORY_ORDER_RELAXED,
    BRIG_MEMORY_ORDER_SC_ACQUIRE,
    BRIG_MEMORY_ORDER_SC_ACQUIRE_RELEASE,
    BRIG_MEMORY_ORDER_SC_RELEASE
};

unsigned InstValidator::MEMSCP_VALUES_WV_WG[] = {
    BRIG_MEMORY_SCOPE_WAVEFRONT,
    BRIG_MEMORY_SCOPE_WORKGROUP
};

unsigned InstValidator::OPERAND_VALUES_ADDRSEG[] = {
    OPERAND_VAL_ADDR
};

unsigned InstValidator::OPERAND_VALUES_IMM[] = {
    OPERAND_VAL_IMM
};

unsigned InstValidator::OPERAND_VALUES_REG_IMM[] = {
    OPERAND_VAL_IMM,
    OPERAND_VAL_REG
};

unsigned InstValidator::OPERAND_VALUES_REG_VECTOR_IMM[] = {
    OPERAND_VAL_IMM,
    OPERAND_VAL_REG,
    OPERAND_VAL_VEC_2,
    OPERAND_VAL_VEC_3,
    OPERAND_VAL_VEC_4
};

unsigned InstValidator::OPERAND_VALUES_NULL[] = {
    OPERAND_VAL_NULL
};

unsigned InstValidator::OPERAND_VALUES_REG[] = {
    OPERAND_VAL_REG
};

unsigned InstValidator::OPERAND_VALUES_REG_VECTOR[] = {
    OPERAND_VAL_REG,
    OPERAND_VAL_VEC_2,
    OPERAND_VAL_VEC_3,
    OPERAND_VAL_VEC_4
};

unsigned InstValidator::OPERAND_VALUES_VEC4[] = {
    OPERAND_VAL_VEC_4
};

unsigned InstValidator::PACK_VALUES_NONE[] = {
    BRIG_PACK_NONE
};

unsigned InstValidator::ROUND_VALUES_NONE[] = {
    BRIG_ROUND_NONE
};

unsigned InstValidator::SEGMENT_VALUES_GCN[] = {
    BRIG_SEGMENT_AMD_GCN
};

unsigned InstValidator::SEGMENT_VALUES_GROUP[] = {
    BRIG_SEGMENT_GROUP
};

unsigned InstValidator::TYPE_VALUES_U_S_F_B128[] = {
    BRIG_TYPE_B128,
    BRIG_TYPE_F16,
    BRIG_TYPE_F32,
    BRIG_TYPE_F64,
    BRIG_TYPE_S16,
    BRIG_TYPE_S32,
    BRIG_TYPE_S64,
    BRIG_TYPE_S8,
    BRIG_TYPE_U16,
    BRIG_TYPE_U32,
    BRIG_TYPE_U64,
    BRIG_TYPE_U8
};

unsigned InstValidator::TYPE_VALUES_B32[] = {
    BRIG_TYPE_B32
};

unsigned InstValidator::TYPE_VALUES_B64[] = {
    BRIG_TYPE_B64
};

unsigned InstValidator::TYPE_VALUES_F[] = {
    BRIG_TYPE_F16,
    BRIG_TYPE_F32,
    BRIG_TYPE_F64
};

unsigned InstValidator::TYPE_VALUES_S32_U32_F[] = {
    BRIG_TYPE_F16,
    BRIG_TYPE_F32,
    BRIG_TYPE_F64,
    BRIG_TYPE_S32,
    BRIG_TYPE_U32
};

unsigned InstValidator::TYPE_VALUES_F32[] = {
    BRIG_TYPE_F32
};

unsigned InstValidator::TYPE_VALUES_F32_F64[] = {
    BRIG_TYPE_F32,
    BRIG_TYPE_F64
};

unsigned InstValidator::TYPE_VALUES_S32_U32_F32[] = {
    BRIG_TYPE_F32,
    BRIG_TYPE_S32,
    BRIG_TYPE_U32
};

unsigned InstValidator::TYPE_VALUES_F64[] = {
    BRIG_TYPE_F64
};

unsigned InstValidator::TYPE_VALUES_U32[] = {
    BRIG_TYPE_U32
};

unsigned InstValidator::TYPESIZE_VALUES_ATOMIC[] = {
    TYPESIZE_VAL_ATOMIC
};

unsigned InstValidator::WIDTH_VALUES_ANY1[] = {
    BRIG_WIDTH_1024,
    BRIG_WIDTH_1048576,
    BRIG_WIDTH_1073741824,
    BRIG_WIDTH_128,
    BRIG_WIDTH_131072,
    BRIG_WIDTH_134217728,
    BRIG_WIDTH_16384,
    BRIG_WIDTH_16777216,
    BRIG_WIDTH_16,
    BRIG_WIDTH_1,
    BRIG_WIDTH_2048,
    BRIG_WIDTH_2097152,
    BRIG_WIDTH_2147483648,
    BRIG_WIDTH_256,
    BRIG_WIDTH_262144,
    BRIG_WIDTH_268435456,
    BRIG_WIDTH_2,
    BRIG_WIDTH_32768,
    BRIG_WIDTH_32,
    BRIG_WIDTH_33554432,
    BRIG_WIDTH_4096,
    BRIG_WIDTH_4194304,
    BRIG_WIDTH_4,
    BRIG_WIDTH_512,
    BRIG_WIDTH_524288,
    BRIG_WIDTH_536870912,
    BRIG_WIDTH_64,
    BRIG_WIDTH_65536,
    BRIG_WIDTH_67108864,
    BRIG_WIDTH_8192,
    BRIG_WIDTH_8388608,
    BRIG_WIDTH_8,
    BRIG_WIDTH_ALL,
    BRIG_WIDTH_WAVESIZE
};

unsigned InstValidator::WIDTH_VALUES_NONE[] = {
    BRIG_WIDTH_NONE
};

bool InstValidator::check_align_values_any(unsigned val)
{
    switch(val)
    {
    case BRIG_ALIGNMENT_1:
    case BRIG_ALIGNMENT_128:
    case BRIG_ALIGNMENT_16:
    case BRIG_ALIGNMENT_2:
    case BRIG_ALIGNMENT_256:
    case BRIG_ALIGNMENT_32:
    case BRIG_ALIGNMENT_4:
    case BRIG_ALIGNMENT_64:
    case BRIG_ALIGNMENT_8:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_atmop_values_add_min_max(unsigned val)
{
    switch(val)
    {
    case BRIG_ATOMIC_ADD:
    case BRIG_ATOMIC_MAX:
    case BRIG_ATOMIC_MIN:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_atmop_values_min_max(unsigned val)
{
    switch(val)
    {
    case BRIG_ATOMIC_MAX:
    case BRIG_ATOMIC_MIN:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_const_values_none(unsigned val)
{
    switch(val)
    {
    case 0:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_ftz_values_any(unsigned val)
{
    switch(val)
    {
    case 0:
    case 1:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_memord_values_any(unsigned val)
{
    switch(val)
    {
    case BRIG_MEMORY_ORDER_RELAXED:
    case BRIG_MEMORY_ORDER_SC_ACQUIRE:
    case BRIG_MEMORY_ORDER_SC_ACQUIRE_RELEASE:
    case BRIG_MEMORY_ORDER_SC_RELEASE:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_memscp_values_wv_wg(unsigned val)
{
    switch(val)
    {
    case BRIG_MEMORY_SCOPE_WAVEFRONT:
    case BRIG_MEMORY_SCOPE_WORKGROUP:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_pack_values_none(unsigned val)
{
    switch(val)
    {
    case BRIG_PACK_NONE:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_round_values_none(unsigned val)
{
    switch(val)
    {
    case BRIG_ROUND_NONE:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_segment_values_gcn(unsigned val)
{
    switch(val)
    {
    case BRIG_SEGMENT_AMD_GCN:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_segment_values_group(unsigned val)
{
    switch(val)
    {
    case BRIG_SEGMENT_GROUP:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_type_values_u_s_f_b128(unsigned val)
{
    switch(val)
    {
    case BRIG_TYPE_B128:
    case BRIG_TYPE_F16:
    case BRIG_TYPE_F32:
    case BRIG_TYPE_F64:
    case BRIG_TYPE_S16:
    case BRIG_TYPE_S32:
    case BRIG_TYPE_S64:
    case BRIG_TYPE_S8:
    case BRIG_TYPE_U16:
    case BRIG_TYPE_U32:
    case BRIG_TYPE_U64:
    case BRIG_TYPE_U8:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_type_values_b32(unsigned val)
{
    switch(val)
    {
    case BRIG_TYPE_B32:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_type_values_b64(unsigned val)
{
    switch(val)
    {
    case BRIG_TYPE_B64:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_type_values_f(unsigned val)
{
    switch(val)
    {
    case BRIG_TYPE_F16:
    case BRIG_TYPE_F32:
    case BRIG_TYPE_F64:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_type_values_s32_u32_f(unsigned val)
{
    switch(val)
    {
    case BRIG_TYPE_F16:
    case BRIG_TYPE_F32:
    case BRIG_TYPE_F64:
    case BRIG_TYPE_S32:
    case BRIG_TYPE_U32:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_type_values_f32(unsigned val)
{
    switch(val)
    {
    case BRIG_TYPE_F32:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_type_values_f32_f64(unsigned val)
{
    switch(val)
    {
    case BRIG_TYPE_F32:
    case BRIG_TYPE_F64:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_type_values_s32_u32_f32(unsigned val)
{
    switch(val)
    {
    case BRIG_TYPE_F32:
    case BRIG_TYPE_S32:
    case BRIG_TYPE_U32:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_type_values_f64(unsigned val)
{
    switch(val)
    {
    case BRIG_TYPE_F64:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_type_values_u32(unsigned val)
{
    switch(val)
    {
    case BRIG_TYPE_U32:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_width_values_any1(unsigned val)
{
    switch(val)
    {
    case BRIG_WIDTH_1024:
    case BRIG_WIDTH_1048576:
    case BRIG_WIDTH_1073741824:
    case BRIG_WIDTH_128:
    case BRIG_WIDTH_131072:
    case BRIG_WIDTH_134217728:
    case BRIG_WIDTH_16384:
    case BRIG_WIDTH_16777216:
    case BRIG_WIDTH_16:
    case BRIG_WIDTH_1:
    case BRIG_WIDTH_2048:
    case BRIG_WIDTH_2097152:
    case BRIG_WIDTH_2147483648:
    case BRIG_WIDTH_256:
    case BRIG_WIDTH_262144:
    case BRIG_WIDTH_268435456:
    case BRIG_WIDTH_2:
    case BRIG_WIDTH_32768:
    case BRIG_WIDTH_32:
    case BRIG_WIDTH_33554432:
    case BRIG_WIDTH_4096:
    case BRIG_WIDTH_4194304:
    case BRIG_WIDTH_4:
    case BRIG_WIDTH_512:
    case BRIG_WIDTH_524288:
    case BRIG_WIDTH_536870912:
    case BRIG_WIDTH_64:
    case BRIG_WIDTH_65536:
    case BRIG_WIDTH_67108864:
    case BRIG_WIDTH_8192:
    case BRIG_WIDTH_8388608:
    case BRIG_WIDTH_8:
    case BRIG_WIDTH_ALL:
    case BRIG_WIDTH_WAVESIZE:
        return true;
    default:
        return false;
    }
}

bool InstValidator::check_width_values_none(unsigned val)
{
    switch(val)
    {
    case BRIG_WIDTH_NONE:
        return true;
    default:
        return false;
    }
}

//================================================================================
//  Req d0_s1 = {
//      d0 = reg;
//      s1 = reg, imm;
//      s2 = null;
//      s3 = null;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_d0_s1(T inst) const
{
    validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req d0_s1_s2 = {
//      d0 = reg;
//      s1 = reg, imm;
//      s2 = reg, imm;
//      s3 = null;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_d0_s1_s2(T inst) const
{
    validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req d0_s1_s2_s3 = {
//      d0 = reg;
//      s1 = reg, imm;
//      s2 = reg, imm;
//      s3 = reg, imm;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_d0_s1_s2_s3(T inst) const
{
    validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req gcn_append_consume = {
//      type = u32;
//      segment = gcn;
//      d0 = reg;
//      s1 = addr_seg;
//      s2 = null;
//      s3 = null;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_gcn_append_consume(T inst) const
{
    if (!check_type_values_u32(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_U32, sizeof(TYPE_VALUES_U32) / sizeof(unsigned));
    }
    if (!check_segment_values_gcn(getSegment<T>(inst))) {
        brigPropError(inst, PROP_SEGMENT, getSegment<T>(inst), SEGMENT_VALUES_GCN, sizeof(SEGMENT_VALUES_GCN) / sizeof(unsigned));
    }
    validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_SEG, OPERAND_VALUES_ADDRSEG, sizeof(OPERAND_VALUES_ADDRSEG) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req gcn_atomic = {
//      type = f32, f64;
//      {type = f32; ? atmop = add, min, max;}
//      {type = f64; ? atmop = min, max;}
//      ;
//      segment = group;
//      memord = any;
//      memscp = wv, wg;
//      eqclass = any;
//      typesize = atomic;
//      d0 = reg;
//      s1 = addr_seg;
//      s2 = reg, imm;
//      s3 = null;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_gcn_atomic(T inst) const
{
    if (!check_type_values_f32_f64(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_F32_F64, sizeof(TYPE_VALUES_F32_F64) / sizeof(unsigned));
    }

    if (
            check_type_values_f32(getType<T>(inst))
       )
    {
        if (!check_atmop_values_add_min_max(getAtomicOperation<T>(inst))) {
            brigPropError(inst, PROP_ATOMICOPERATION, getAtomicOperation<T>(inst), ATMOP_VALUES_ADD_MIN_MAX, sizeof(ATMOP_VALUES_ADD_MIN_MAX) / sizeof(unsigned));
        }
    }
    else if (
            check_type_values_f64(getType<T>(inst))
       )
    {
        if (!check_atmop_values_min_max(getAtomicOperation<T>(inst))) {
            brigPropError(inst, PROP_ATOMICOPERATION, getAtomicOperation<T>(inst), ATMOP_VALUES_MIN_MAX, sizeof(ATMOP_VALUES_MIN_MAX) / sizeof(unsigned));
        }
    }
    else
    {
        invalidVariant(inst, PROP_TYPE);
    }
    if (!check_segment_values_group(getSegment<T>(inst))) {
        brigPropError(inst, PROP_SEGMENT, getSegment<T>(inst), SEGMENT_VALUES_GROUP, sizeof(SEGMENT_VALUES_GROUP) / sizeof(unsigned));
    }
    if (!check_memord_values_any(getMemoryOrder<T>(inst))) {
        brigPropError(inst, PROP_MEMORYORDER, getMemoryOrder<T>(inst), MEMORD_VALUES_ANY, sizeof(MEMORD_VALUES_ANY) / sizeof(unsigned));
    }
    if (!check_memscp_values_wv_wg(getMemoryScope<T>(inst))) {
        brigPropError(inst, PROP_MEMORYSCOPE, getMemoryScope<T>(inst), MEMSCP_VALUES_WV_WG, sizeof(MEMSCP_VALUES_WV_WG) / sizeof(unsigned));
    }
    validateEqclass(inst, PROP_EQUIVCLASS, EQCLASS_ATTR_NONE, EQCLASS_VALUES_ANY, sizeof(EQCLASS_VALUES_ANY) / sizeof(unsigned));
    validateTypesize(inst, PROP_TYPESIZE, TYPESIZE_ATTR_NONE, TYPESIZE_VALUES_ATOMIC, sizeof(TYPESIZE_VALUES_ATOMIC) / sizeof(unsigned));
    validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_SEG, OPERAND_VALUES_ADDRSEG, sizeof(OPERAND_VALUES_ADDRSEG) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req gcn_atomic_noret = {
//      type = f32, f64;
//      {type = f32; ? atmop = add, min, max;}
//      {type = f64; ? atmop = min, max;}
//      ;
//      segment = group;
//      memord = any;
//      memscp = wv, wg;
//      eqclass = any;
//      typesize = atomic;
//      s0 = addr_seg;
//      s1 = reg, imm;
//      s2 = null;
//      s3 = null;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_gcn_atomic_noret(T inst) const
{
    if (!check_type_values_f32_f64(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_F32_F64, sizeof(TYPE_VALUES_F32_F64) / sizeof(unsigned));
    }

    if (
            check_type_values_f32(getType<T>(inst))
       )
    {
        if (!check_atmop_values_add_min_max(getAtomicOperation<T>(inst))) {
            brigPropError(inst, PROP_ATOMICOPERATION, getAtomicOperation<T>(inst), ATMOP_VALUES_ADD_MIN_MAX, sizeof(ATMOP_VALUES_ADD_MIN_MAX) / sizeof(unsigned));
        }
    }
    else if (
            check_type_values_f64(getType<T>(inst))
       )
    {
        if (!check_atmop_values_min_max(getAtomicOperation<T>(inst))) {
            brigPropError(inst, PROP_ATOMICOPERATION, getAtomicOperation<T>(inst), ATMOP_VALUES_MIN_MAX, sizeof(ATMOP_VALUES_MIN_MAX) / sizeof(unsigned));
        }
    }
    else
    {
        invalidVariant(inst, PROP_TYPE);
    }
    if (!check_segment_values_group(getSegment<T>(inst))) {
        brigPropError(inst, PROP_SEGMENT, getSegment<T>(inst), SEGMENT_VALUES_GROUP, sizeof(SEGMENT_VALUES_GROUP) / sizeof(unsigned));
    }
    if (!check_memord_values_any(getMemoryOrder<T>(inst))) {
        brigPropError(inst, PROP_MEMORYORDER, getMemoryOrder<T>(inst), MEMORD_VALUES_ANY, sizeof(MEMORD_VALUES_ANY) / sizeof(unsigned));
    }
    if (!check_memscp_values_wv_wg(getMemoryScope<T>(inst))) {
        brigPropError(inst, PROP_MEMORYSCOPE, getMemoryScope<T>(inst), MEMSCP_VALUES_WV_WG, sizeof(MEMSCP_VALUES_WV_WG) / sizeof(unsigned));
    }
    validateEqclass(inst, PROP_EQUIVCLASS, EQCLASS_ATTR_NONE, EQCLASS_VALUES_ANY, sizeof(EQCLASS_VALUES_ANY) / sizeof(unsigned));
    validateTypesize(inst, PROP_TYPESIZE, TYPESIZE_ATTR_NONE, TYPESIZE_VALUES_ATOMIC, sizeof(TYPESIZE_VALUES_ATOMIC) / sizeof(unsigned));
    validateOperand(inst, PROP_S0, OPERAND_ATTR_SEG, OPERAND_VALUES_ADDRSEG, sizeof(OPERAND_VALUES_ADDRSEG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req gcn_b4xchg = {
//      type = b32;
//      d0 = reg;
//      s1 = reg;
//      s2 = imm;
//      s3 = null;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_gcn_b4xchg(T inst) const
{
    if (!check_type_values_b32(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_B32, sizeof(TYPE_VALUES_B32) / sizeof(unsigned));
    }
    validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_DTYPE, OPERAND_VALUES_IMM, sizeof(OPERAND_VALUES_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req gcn_bfm = {
//      type = b32;
//      d0 = reg;
//      s1 = reg, imm;
//      s2 = reg, imm;
//      s3 = null;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_gcn_bfm(T inst) const
{
    if (!check_type_values_b32(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_B32, sizeof(TYPE_VALUES_B32) / sizeof(unsigned));
    }
    validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req gcn_div_relaxed = {
//      type = f32;
//      pack = none;
//      round = none;
//      ftz = any;
//      d0_s1_s2;
//  }
template<class T> bool InstValidator::req_gcn_div_relaxed(T inst) const
{
    if (!check_type_values_f32(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_F32, sizeof(TYPE_VALUES_F32) / sizeof(unsigned));
    }
    if (!check_pack_values_none(getPackEx<T>(inst))) {
        brigPropError(inst, PROP_PACK, getPackEx<T>(inst), PACK_VALUES_NONE, sizeof(PACK_VALUES_NONE) / sizeof(unsigned));
    }
    validateRound(inst, PROP_ROUND, getRoundEx<T>(inst), ROUND_VALUES_NONE, sizeof(ROUND_VALUES_NONE) / sizeof(unsigned));
    validateFtz(inst, PROP_FTZ, getFtzEx<T>(inst), FTZ_VALUES_ANY, sizeof(FTZ_VALUES_ANY) / sizeof(unsigned));
    req_d0_s1_s2(inst);
    return true;
}

//================================================================================
//  Req gcn_div_relaxed_narrow = {
//      type = f32;
//      d0_s1_s2;
//  }
template<class T> bool InstValidator::req_gcn_div_relaxed_narrow(T inst) const
{
    if (!check_type_values_f32(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_F32, sizeof(TYPE_VALUES_F32) / sizeof(unsigned));
    }
    req_d0_s1_s2(inst);
    return true;
}

//================================================================================
//  Req gcn_fldexp = {
//      type = f;
//      pack = none;
//      round = none;
//      ftz = any;
//      d0 = reg;
//      s1 = reg, imm;
//      s2 = reg_s32, imm_s32;
//      s3 = null;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_gcn_fldexp(T inst) const
{
    if (!check_type_values_f(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_F, sizeof(TYPE_VALUES_F) / sizeof(unsigned));
    }
    if (!check_pack_values_none(getPackEx<T>(inst))) {
        brigPropError(inst, PROP_PACK, getPackEx<T>(inst), PACK_VALUES_NONE, sizeof(PACK_VALUES_NONE) / sizeof(unsigned));
    }
    validateRound(inst, PROP_ROUND, getRoundEx<T>(inst), ROUND_VALUES_NONE, sizeof(ROUND_VALUES_NONE) / sizeof(unsigned));
    validateFtz(inst, PROP_FTZ, getFtzEx<T>(inst), FTZ_VALUES_ANY, sizeof(FTZ_VALUES_ANY) / sizeof(unsigned));
    validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_S32, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req gcn_frexp_exp = {
//      type = f;
//      pack = none;
//      round = none;
//      ftz = any;
//      d0 = reg_s32;
//      s1 = reg, imm;
//      s2 = null;
//      s3 = null;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_gcn_frexp_exp(T inst) const
{
    if (!check_type_values_f(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_F, sizeof(TYPE_VALUES_F) / sizeof(unsigned));
    }
    if (!check_pack_values_none(getPackEx<T>(inst))) {
        brigPropError(inst, PROP_PACK, getPackEx<T>(inst), PACK_VALUES_NONE, sizeof(PACK_VALUES_NONE) / sizeof(unsigned));
    }
    validateRound(inst, PROP_ROUND, getRoundEx<T>(inst), ROUND_VALUES_NONE, sizeof(ROUND_VALUES_NONE) / sizeof(unsigned));
    validateFtz(inst, PROP_FTZ, getFtzEx<T>(inst), FTZ_VALUES_ANY, sizeof(FTZ_VALUES_ANY) / sizeof(unsigned));
    validateOperand(inst, PROP_D0, OPERAND_ATTR_S32, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req gcn_frexp_mant = {
//      type = f;
//      pack = none;
//      round = none;
//      ftz = any;
//      d0 = reg;
//      s1 = reg, imm;
//      s2 = null;
//      s3 = null;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_gcn_frexp_mant(T inst) const
{
    if (!check_type_values_f(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_F, sizeof(TYPE_VALUES_F) / sizeof(unsigned));
    }
    if (!check_pack_values_none(getPackEx<T>(inst))) {
        brigPropError(inst, PROP_PACK, getPackEx<T>(inst), PACK_VALUES_NONE, sizeof(PACK_VALUES_NONE) / sizeof(unsigned));
    }
    validateRound(inst, PROP_ROUND, getRoundEx<T>(inst), ROUND_VALUES_NONE, sizeof(ROUND_VALUES_NONE) / sizeof(unsigned));
    validateFtz(inst, PROP_FTZ, getFtzEx<T>(inst), FTZ_VALUES_ANY, sizeof(FTZ_VALUES_ANY) / sizeof(unsigned));
    validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req gcn_ld = {
//      type = u, s, f, b128;
//      segment = gcn;
//      align = any;
//      const = none;
//      eqclass = any;
//      width = any_1;
//      d0 = reg, vector;
//      s1 = addr_seg;
//      s2 = null;
//      s3 = null;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_gcn_ld(T inst) const
{
    if (!check_type_values_u_s_f_b128(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_U_S_F_B128, sizeof(TYPE_VALUES_U_S_F_B128) / sizeof(unsigned));
    }
    if (!check_segment_values_gcn(getSegment<T>(inst))) {
        brigPropError(inst, PROP_SEGMENT, getSegment<T>(inst), SEGMENT_VALUES_GCN, sizeof(SEGMENT_VALUES_GCN) / sizeof(unsigned));
    }
    if (!check_align_values_any(getAlign<T>(inst))) {
        brigPropError(inst, PROP_ALIGN, getAlign<T>(inst), ALIGN_VALUES_ANY, sizeof(ALIGN_VALUES_ANY) / sizeof(unsigned));
    }
    if (!check_const_values_none(getIsConst<T>(inst))) {
        brigPropError(inst, PROP_ISCONST, getIsConst<T>(inst), CONST_VALUES_NONE, sizeof(CONST_VALUES_NONE) / sizeof(unsigned));
    }
    validateEqclass(inst, PROP_EQUIVCLASS, EQCLASS_ATTR_NONE, EQCLASS_VALUES_ANY, sizeof(EQCLASS_VALUES_ANY) / sizeof(unsigned));
    if (!check_width_values_any1(getWidth<T>(inst))) {
        brigPropError(inst, PROP_WIDTH, getWidth<T>(inst), WIDTH_VALUES_ANY1, sizeof(WIDTH_VALUES_ANY1) / sizeof(unsigned));
    }
    validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_VECTOR, sizeof(OPERAND_VALUES_REG_VECTOR) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_SEG, OPERAND_VALUES_ADDRSEG, sizeof(OPERAND_VALUES_ADDRSEG) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req gcn_mads = {
//      type = b32;
//      d0 = reg_s64;
//      s1 = reg_s32, imm_s32;
//      s2 = reg_s32, imm_s32;
//      s3 = reg_s64, imm_s64;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_gcn_mads(T inst) const
{
    if (!check_type_values_b32(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_B32, sizeof(TYPE_VALUES_B32) / sizeof(unsigned));
    }
    validateOperand(inst, PROP_D0, OPERAND_ATTR_S64, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_S32, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_S32, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_S64, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req gcn_madu = {
//      type = b32;
//      d0 = reg_u64;
//      s1 = reg_u32, imm_u32;
//      s2 = reg_u32, imm_u32;
//      s3 = reg_u64, imm_u64;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_gcn_madu(T inst) const
{
    if (!check_type_values_b32(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_B32, sizeof(TYPE_VALUES_B32) / sizeof(unsigned));
    }
    validateOperand(inst, PROP_D0, OPERAND_ATTR_U64, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_U32, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_U32, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_U64, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req gcn_min_max = {
//      type = s32, u32, f;
//      d0_s1_s2;
//  }
template<class T> bool InstValidator::req_gcn_min_max(T inst) const
{
    if (!check_type_values_s32_u32_f(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_S32_U32_F, sizeof(TYPE_VALUES_S32_U32_F) / sizeof(unsigned));
    }
    req_d0_s1_s2(inst);
    return true;
}

//================================================================================
//  Req gcn_min_max_med3 = {
//      type = s32, u32, f32;
//      d0 = reg;
//      s1 = reg, imm;
//      s2 = reg, imm;
//      s3 = reg, imm;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_gcn_min_max_med3(T inst) const
{
    if (!check_type_values_s32_u32_f32(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_S32_U32_F32, sizeof(TYPE_VALUES_S32_U32_F32) / sizeof(unsigned));
    }
    validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req gcn_mqsad = {
//      type = b64;
//      d0 = reg;
//      s1 = reg, imm;
//      s2 = reg_b32, imm_b32;
//      s3 = reg;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_gcn_mqsad(T inst) const
{
    if (!check_type_values_b64(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_B64, sizeof(TYPE_VALUES_B64) / sizeof(unsigned));
    }
    validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_B32, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req gcn_mqsad4 = {
//      type = b32;
//      d0 = vec_4;
//      s1 = reg_b64, imm_b64;
//      s2 = reg, imm;
//      s3 = vec_4;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_gcn_mqsad4(T inst) const
{
    if (!check_type_values_b32(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_B32, sizeof(TYPE_VALUES_B32) / sizeof(unsigned));
    }
    validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_VEC4, sizeof(OPERAND_VALUES_VEC4) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_B64, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_DTYPE, OPERAND_VALUES_VEC4, sizeof(OPERAND_VALUES_VEC4) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req gcn_msad = {
//      type = b32;
//      d0 = reg;
//      s1 = reg, imm;
//      s2 = reg, imm;
//      s3 = reg, imm;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_gcn_msad(T inst) const
{
    if (!check_type_values_b32(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_B32, sizeof(TYPE_VALUES_B32) / sizeof(unsigned));
    }
    validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req gcn_qsad = {
//      type = b64;
//      d0 = reg;
//      s1 = reg, imm;
//      s2 = reg_b32, imm_b32;
//      s3 = reg, imm;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_gcn_qsad(T inst) const
{
    if (!check_type_values_b64(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_B64, sizeof(TYPE_VALUES_B64) / sizeof(unsigned));
    }
    validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_B32, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req gcn_region_alloc = {
//      type = b32;
//      s0 = imm_u32;
//      s1 = null;
//      s2 = null;
//      s3 = null;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_gcn_region_alloc(T inst) const
{
    if (!check_type_values_b32(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_B32, sizeof(TYPE_VALUES_B32) / sizeof(unsigned));
    }
    validateOperand(inst, PROP_S0, OPERAND_ATTR_U32, OPERAND_VALUES_IMM, sizeof(OPERAND_VALUES_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req gcn_slp_prt = {
//      type = u32;
//      s0 = reg, imm;
//      s1 = null;
//      s2 = null;
//      s3 = null;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_gcn_slp_prt(T inst) const
{
    if (!check_type_values_u32(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_U32, sizeof(TYPE_VALUES_U32) / sizeof(unsigned));
    }
    validateOperand(inst, PROP_S0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req gcn_st = {
//      type = u, s, f, b128;
//      segment = gcn;
//      align = any;
//      const = none;
//      eqclass = any;
//      width = none;
//      s0 = reg, vector, imm;
//      s1 = addr_seg;
//      s2 = null;
//      s3 = null;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_gcn_st(T inst) const
{
    if (!check_type_values_u_s_f_b128(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_U_S_F_B128, sizeof(TYPE_VALUES_U_S_F_B128) / sizeof(unsigned));
    }
    if (!check_segment_values_gcn(getSegment<T>(inst))) {
        brigPropError(inst, PROP_SEGMENT, getSegment<T>(inst), SEGMENT_VALUES_GCN, sizeof(SEGMENT_VALUES_GCN) / sizeof(unsigned));
    }
    if (!check_align_values_any(getAlign<T>(inst))) {
        brigPropError(inst, PROP_ALIGN, getAlign<T>(inst), ALIGN_VALUES_ANY, sizeof(ALIGN_VALUES_ANY) / sizeof(unsigned));
    }
    if (!check_const_values_none(getIsConst<T>(inst))) {
        brigPropError(inst, PROP_ISCONST, getIsConst<T>(inst), CONST_VALUES_NONE, sizeof(CONST_VALUES_NONE) / sizeof(unsigned));
    }
    validateEqclass(inst, PROP_EQUIVCLASS, EQCLASS_ATTR_NONE, EQCLASS_VALUES_ANY, sizeof(EQCLASS_VALUES_ANY) / sizeof(unsigned));
    if (!check_width_values_none(getWidth<T>(inst))) {
        brigPropError(inst, PROP_WIDTH, getWidth<T>(inst), WIDTH_VALUES_NONE, sizeof(WIDTH_VALUES_NONE) / sizeof(unsigned));
    }
    validateOperand(inst, PROP_S0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_VECTOR_IMM, sizeof(OPERAND_VALUES_REG_VECTOR_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_SEG, OPERAND_VALUES_ADDRSEG, sizeof(OPERAND_VALUES_ADDRSEG) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

//================================================================================
//  Req gcn_trig_preop = {
//      type = f64;
//      pack = none;
//      round = none;
//      ftz = any;
//      d0 = reg;
//      s1 = reg, imm;
//      s2 = reg_u32, imm_u32;
//      s3 = null;
//      s4 = null;
//      s5 = null;
//  }
template<class T> bool InstValidator::req_gcn_trig_preop(T inst) const
{
    if (!check_type_values_f64(getType<T>(inst))) {
        brigPropError(inst, PROP_TYPE, getType<T>(inst), TYPE_VALUES_F64, sizeof(TYPE_VALUES_F64) / sizeof(unsigned));
    }
    if (!check_pack_values_none(getPackEx<T>(inst))) {
        brigPropError(inst, PROP_PACK, getPackEx<T>(inst), PACK_VALUES_NONE, sizeof(PACK_VALUES_NONE) / sizeof(unsigned));
    }
    validateRound(inst, PROP_ROUND, getRoundEx<T>(inst), ROUND_VALUES_NONE, sizeof(ROUND_VALUES_NONE) / sizeof(unsigned));
    validateFtz(inst, PROP_FTZ, getFtzEx<T>(inst), FTZ_VALUES_ANY, sizeof(FTZ_VALUES_ANY) / sizeof(unsigned));
    validateOperand(inst, PROP_D0, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG, sizeof(OPERAND_VALUES_REG) / sizeof(unsigned));
    validateOperand(inst, PROP_S1, OPERAND_ATTR_DTYPE, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S2, OPERAND_ATTR_U32, OPERAND_VALUES_REG_IMM, sizeof(OPERAND_VALUES_REG_IMM) / sizeof(unsigned));
    validateOperand(inst, PROP_S3, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S4, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    validateOperand(inst, PROP_S5, OPERAND_ATTR_NONE, OPERAND_VALUES_NULL, sizeof(OPERAND_VALUES_NULL) / sizeof(unsigned));
    return true;
}

void InstValidator::validateInst(Inst inst) const
{
    switch (inst.opcode())
    {
        case (BRIG_OPCODE_AMD_GCN_APPEND):
        {
            InstAddr i = inst;
            if (!i) { invalidFormat(inst, "InstAddr"); }
            req_gcn_append_consume<InstAddr>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_ATOMIC):
        {
            InstAtomic i = inst;
            if (!i) { invalidFormat(inst, "InstAtomic"); }
            req_gcn_atomic<InstAtomic>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_ATOMICNORET):
        {
            InstAtomic i = inst;
            if (!i) { invalidFormat(inst, "InstAtomic"); }
            req_gcn_atomic_noret<InstAtomic>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_B32XCHG):
        {
            InstBasic i = inst;
            if (!i) { invalidFormat(inst, "InstBasic"); }
            req_gcn_b4xchg<InstBasic>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_B4XCHG):
        {
            InstBasic i = inst;
            if (!i) { invalidFormat(inst, "InstBasic"); }
            req_gcn_b4xchg<InstBasic>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_BFM):
        {
            InstBasic i = inst;
            if (!i) { invalidFormat(inst, "InstBasic"); }
            req_gcn_bfm<InstBasic>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_CONSUME):
        {
            InstAddr i = inst;
            if (!i) { invalidFormat(inst, "InstAddr"); }
            req_gcn_append_consume<InstAddr>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_DIVRELAXED):
        {
            if (InstMod i = inst)
            {
                req_gcn_div_relaxed<InstMod>(i);
            }
            else if (InstBasic i = inst)
            {
                req_gcn_div_relaxed<InstBasic>(i);
            }
            else
            {
                invalidFormat(inst, "InstBasic or InstMod");
            }
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_DIVRELAXEDNARROW):
        {
            InstBasic i = inst;
            if (!i) { invalidFormat(inst, "InstBasic"); }
            req_gcn_div_relaxed_narrow<InstBasic>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_FLDEXP):
        {
            if (InstMod i = inst)
            {
                req_gcn_fldexp<InstMod>(i);
            }
            else if (InstBasic i = inst)
            {
                req_gcn_fldexp<InstBasic>(i);
            }
            else
            {
                invalidFormat(inst, "InstBasic or InstMod");
            }
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_FREXP_EXP):
        {
            if (InstMod i = inst)
            {
                req_gcn_frexp_exp<InstMod>(i);
            }
            else if (InstBasic i = inst)
            {
                req_gcn_frexp_exp<InstBasic>(i);
            }
            else
            {
                invalidFormat(inst, "InstBasic or InstMod");
            }
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_FREXP_MANT):
        {
            if (InstMod i = inst)
            {
                req_gcn_frexp_mant<InstMod>(i);
            }
            else if (InstBasic i = inst)
            {
                req_gcn_frexp_mant<InstBasic>(i);
            }
            else
            {
                invalidFormat(inst, "InstBasic or InstMod");
            }
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_LD):
        {
            InstMem i = inst;
            if (!i) { invalidFormat(inst, "InstMem"); }
            req_gcn_ld<InstMem>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_MADS):
        {
            InstBasic i = inst;
            if (!i) { invalidFormat(inst, "InstBasic"); }
            req_gcn_mads<InstBasic>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_MADU):
        {
            InstBasic i = inst;
            if (!i) { invalidFormat(inst, "InstBasic"); }
            req_gcn_madu<InstBasic>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_MAX):
        {
            InstBasic i = inst;
            if (!i) { invalidFormat(inst, "InstBasic"); }
            req_gcn_min_max<InstBasic>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_MAX3):
        {
            InstBasic i = inst;
            if (!i) { invalidFormat(inst, "InstBasic"); }
            req_gcn_min_max_med3<InstBasic>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_MED3):
        {
            InstBasic i = inst;
            if (!i) { invalidFormat(inst, "InstBasic"); }
            req_gcn_min_max_med3<InstBasic>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_MIN):
        {
            InstBasic i = inst;
            if (!i) { invalidFormat(inst, "InstBasic"); }
            req_gcn_min_max<InstBasic>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_MIN3):
        {
            InstBasic i = inst;
            if (!i) { invalidFormat(inst, "InstBasic"); }
            req_gcn_min_max_med3<InstBasic>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_MQSAD):
        {
            InstBasic i = inst;
            if (!i) { invalidFormat(inst, "InstBasic"); }
            req_gcn_mqsad<InstBasic>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_MQSAD4):
        {
            InstBasic i = inst;
            if (!i) { invalidFormat(inst, "InstBasic"); }
            req_gcn_mqsad4<InstBasic>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_MSAD):
        {
            InstBasic i = inst;
            if (!i) { invalidFormat(inst, "InstBasic"); }
            req_gcn_msad<InstBasic>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_PRIORITY):
        {
            InstBasic i = inst;
            if (!i) { invalidFormat(inst, "InstBasic"); }
            req_gcn_slp_prt<InstBasic>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_QSAD):
        {
            InstBasic i = inst;
            if (!i) { invalidFormat(inst, "InstBasic"); }
            req_gcn_qsad<InstBasic>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_REGIONALLOC):
        {
            InstBasic i = inst;
            if (!i) { invalidFormat(inst, "InstBasic"); }
            req_gcn_region_alloc<InstBasic>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_SADD):
        {
            InstBasic i = inst;
            if (!i) { invalidFormat(inst, "InstBasic"); }
            req_gcn_msad<InstBasic>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_SADW):
        {
            InstBasic i = inst;
            if (!i) { invalidFormat(inst, "InstBasic"); }
            req_gcn_msad<InstBasic>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_SLEEP):
        {
            InstBasic i = inst;
            if (!i) { invalidFormat(inst, "InstBasic"); }
            req_gcn_slp_prt<InstBasic>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_ST):
        {
            InstMem i = inst;
            if (!i) { invalidFormat(inst, "InstMem"); }
            req_gcn_st<InstMem>(i);
            break;
        }
        case (BRIG_OPCODE_AMD_GCN_TRIG_PREOP):
        {
            if (InstMod i = inst)
            {
                req_gcn_trig_preop<InstMod>(i);
            }
            else if (InstBasic i = inst)
            {
                req_gcn_trig_preop<InstBasic>(i);
            }
            else
            {
                invalidFormat(inst, "InstBasic or InstMod");
            }
            break;
        }
        default:
            validate(inst, false, "Invalid instruction opcode");
            break;
    }
} // InstValidator::validateInst

unsigned InstValidator::getOperand0Attr(Inst inst) const
{
    switch (inst.opcode())
    {
        case (BRIG_OPCODE_AMD_GCN_APPEND): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_ATOMIC): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_ATOMICNORET): return OPERAND_ATTR_SEG;
        case (BRIG_OPCODE_AMD_GCN_B32XCHG): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_B4XCHG): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_BFM): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_CONSUME): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_DIVRELAXED): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_DIVRELAXEDNARROW): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_FLDEXP): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_FREXP_EXP): return OPERAND_ATTR_S32;
        case (BRIG_OPCODE_AMD_GCN_FREXP_MANT): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_LD): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_MADS): return OPERAND_ATTR_S64;
        case (BRIG_OPCODE_AMD_GCN_MADU): return OPERAND_ATTR_U64;
        case (BRIG_OPCODE_AMD_GCN_MAX): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_MAX3): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_MED3): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_MIN): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_MIN3): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_MQSAD): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_MQSAD4): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_MSAD): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_PRIORITY): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_QSAD): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_REGIONALLOC): return OPERAND_ATTR_U32;
        case (BRIG_OPCODE_AMD_GCN_SADD): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_SADW): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_SLEEP): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_ST): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_TRIG_PREOP): return OPERAND_ATTR_DTYPE;
        default:
            return OPERAND_ATTR_INVALID;
            break;
    }
} // InstValidator::getOperand0Attr

unsigned InstValidator::getOperand1Attr(Inst inst) const
{
    switch (inst.opcode())
    {
        case (BRIG_OPCODE_AMD_GCN_APPEND): return OPERAND_ATTR_SEG;
        case (BRIG_OPCODE_AMD_GCN_ATOMIC): return OPERAND_ATTR_SEG;
        case (BRIG_OPCODE_AMD_GCN_ATOMICNORET): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_B32XCHG): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_B4XCHG): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_BFM): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_CONSUME): return OPERAND_ATTR_SEG;
        case (BRIG_OPCODE_AMD_GCN_DIVRELAXED): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_DIVRELAXEDNARROW): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_FLDEXP): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_FREXP_EXP): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_FREXP_MANT): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_LD): return OPERAND_ATTR_SEG;
        case (BRIG_OPCODE_AMD_GCN_MADS): return OPERAND_ATTR_S32;
        case (BRIG_OPCODE_AMD_GCN_MADU): return OPERAND_ATTR_U32;
        case (BRIG_OPCODE_AMD_GCN_MAX): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_MAX3): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_MED3): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_MIN): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_MIN3): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_MQSAD): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_MQSAD4): return OPERAND_ATTR_B64;
        case (BRIG_OPCODE_AMD_GCN_MSAD): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_PRIORITY): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_QSAD): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_REGIONALLOC): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_SADD): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_SADW): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_SLEEP): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_ST): return OPERAND_ATTR_SEG;
        case (BRIG_OPCODE_AMD_GCN_TRIG_PREOP): return OPERAND_ATTR_DTYPE;
        default:
            return OPERAND_ATTR_INVALID;
            break;
    }
} // InstValidator::getOperand1Attr

unsigned InstValidator::getOperand2Attr(Inst inst) const
{
    switch (inst.opcode())
    {
        case (BRIG_OPCODE_AMD_GCN_APPEND): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_ATOMIC): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_ATOMICNORET): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_B32XCHG): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_B4XCHG): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_BFM): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_CONSUME): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_DIVRELAXED): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_DIVRELAXEDNARROW): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_FLDEXP): return OPERAND_ATTR_S32;
        case (BRIG_OPCODE_AMD_GCN_FREXP_EXP): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_FREXP_MANT): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_LD): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_MADS): return OPERAND_ATTR_S32;
        case (BRIG_OPCODE_AMD_GCN_MADU): return OPERAND_ATTR_U32;
        case (BRIG_OPCODE_AMD_GCN_MAX): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_MAX3): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_MED3): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_MIN): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_MIN3): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_MQSAD): return OPERAND_ATTR_B32;
        case (BRIG_OPCODE_AMD_GCN_MQSAD4): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_MSAD): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_PRIORITY): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_QSAD): return OPERAND_ATTR_B32;
        case (BRIG_OPCODE_AMD_GCN_REGIONALLOC): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_SADD): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_SADW): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_SLEEP): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_ST): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_TRIG_PREOP): return OPERAND_ATTR_U32;
        default:
            return OPERAND_ATTR_INVALID;
            break;
    }
} // InstValidator::getOperand2Attr

unsigned InstValidator::getOperand3Attr(Inst inst) const
{
    switch (inst.opcode())
    {
        case (BRIG_OPCODE_AMD_GCN_APPEND): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_ATOMIC): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_ATOMICNORET): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_B32XCHG): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_B4XCHG): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_BFM): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_CONSUME): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_DIVRELAXED): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_DIVRELAXEDNARROW): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_FLDEXP): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_FREXP_EXP): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_FREXP_MANT): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_LD): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_MADS): return OPERAND_ATTR_S64;
        case (BRIG_OPCODE_AMD_GCN_MADU): return OPERAND_ATTR_U64;
        case (BRIG_OPCODE_AMD_GCN_MAX): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_MAX3): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_MED3): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_MIN): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_MIN3): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_MQSAD): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_MQSAD4): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_MSAD): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_PRIORITY): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_QSAD): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_REGIONALLOC): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_SADD): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_SADW): return OPERAND_ATTR_DTYPE;
        case (BRIG_OPCODE_AMD_GCN_SLEEP): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_ST): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_TRIG_PREOP): return OPERAND_ATTR_NONE;
        default:
            return OPERAND_ATTR_INVALID;
            break;
    }
} // InstValidator::getOperand3Attr

unsigned InstValidator::getOperand4Attr(Inst inst) const
{
    switch (inst.opcode())
    {
        case (BRIG_OPCODE_AMD_GCN_APPEND): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_ATOMIC): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_ATOMICNORET): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_B32XCHG): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_B4XCHG): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_BFM): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_CONSUME): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_DIVRELAXED): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_DIVRELAXEDNARROW): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_FLDEXP): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_FREXP_EXP): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_FREXP_MANT): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_LD): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_MADS): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_MADU): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_MAX): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_MAX3): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_MED3): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_MIN): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_MIN3): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_MQSAD): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_MQSAD4): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_MSAD): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_PRIORITY): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_QSAD): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_REGIONALLOC): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_SADD): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_SADW): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_SLEEP): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_ST): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_TRIG_PREOP): return OPERAND_ATTR_NONE;
        default:
            return OPERAND_ATTR_INVALID;
            break;
    }
} // InstValidator::getOperand4Attr

unsigned InstValidator::getOperand5Attr(Inst inst) const
{
    switch (inst.opcode())
    {
        case (BRIG_OPCODE_AMD_GCN_APPEND): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_ATOMIC): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_ATOMICNORET): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_B32XCHG): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_B4XCHG): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_BFM): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_CONSUME): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_DIVRELAXED): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_DIVRELAXEDNARROW): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_FLDEXP): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_FREXP_EXP): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_FREXP_MANT): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_LD): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_MADS): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_MADU): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_MAX): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_MAX3): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_MED3): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_MIN): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_MIN3): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_MQSAD): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_MQSAD4): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_MSAD): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_PRIORITY): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_QSAD): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_REGIONALLOC): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_SADD): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_SADW): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_SLEEP): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_ST): return OPERAND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_TRIG_PREOP): return OPERAND_ATTR_NONE;
        default:
            return OPERAND_ATTR_INVALID;
            break;
    }
} // InstValidator::getOperand5Attr

unsigned InstValidator::getRoundAttr(Inst inst) const
{
    switch (inst.opcode())
    {
        case (BRIG_OPCODE_AMD_GCN_DIVRELAXED): return ROUND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_FLDEXP): return ROUND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_FREXP_EXP): return ROUND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_FREXP_MANT): return ROUND_ATTR_NONE;
        case (BRIG_OPCODE_AMD_GCN_TRIG_PREOP): return ROUND_ATTR_NONE;
        default:
            return ROUND_ATTR_INVALID;
            break;
    }
} // InstValidator::getRoundAttr

unsigned InstValidator::getWidthAttr(Inst inst) const
{
    switch (inst.opcode())
    {
        case (BRIG_OPCODE_AMD_GCN_LD): return WIDTH_ATTR_1;
        case (BRIG_OPCODE_AMD_GCN_ST): return WIDTH_ATTR_NONE;
        default:
            return WIDTH_ATTR_INVALID;
            break;
    }
} // InstValidator::getWidthAttr

